#!/usr/bin/env python3
"""
Javantea's Reverse Engineering Tool Version 2
by Javantea
Oct 2, 2014

Reverse engineers a program disassembled with objdump

Based on JavRE, Javantea's Reverse Engineer Oct 12, 2010

Parses a file generated from:
objdump -D -b binary -m i386 P8Z77-V-LE-ASUS-1104.CAP>P8Z77-V-LE-ASUS-1104.dis

objdump -D -b binary -m i386 -M x86_64 P8Z77-V-LE-ASUS-1104.CAP>P8Z77-V-LE-ASUS-1104_64.dis

20588 calls found.
3426 funcs found.

TODO:
Make max function address programmatic.
Take any input file.
Use distorm instead of objdump.
Find data sections.
Find image files.
Find other files.

"""

import re
import sys

def usage():
	print("""JavRE
Usage:
	python3 javre2.py filename.dis >filename.rev
""")
#end def usage()

filename = None
if len(sys.argv) > 1:
	filename = sys.argv[1]
else:
	usage()
	exit(1)
#end if

lines = open(filename,'r').read().split('\n')

call_re = re.compile(r'^ +([0-9a-f]+):\t[ 0-9a-f]*\tcallq? +0x([0-9a-f]*)')
#dyn_call_re = re.compile(r'^ +([0-9a-f]+):\t[ 0-9a-f]*\tcallq? +(.*)')
calls = []
for line in lines:
	call = call_re.match(line)
	if call:
		calls.append((call.group(1), call.group(2)))
	#end if
#next line

print(len(calls), "calls found.")

# TODO: Make this programmatic.
max_addr = 0x800800
funcs = [x for x in set([int(c[1], 16) for c in calls]) if x < max_addr]

print(len(funcs), "funcs found.")

# Now we have all the function locations, let's print out the disassembly with the functions pointed out.

addr_re = re.compile(r'^ +([0-9a-f]+):')

for line in lines:
	call = call_re.match(line)
	if call:
		# A call, print the function name.
		addr = int(call.group(1), 16)
		if addr in funcs:
			# Function headers add a lot.
			print('sub_%08x:' % addr)
		#end if
		dest = int(call.group(2), 16)
		if dest in funcs:
			# show them the function name so that they can search/replace it using a text editor without losing the address.
			print(line, '<sub_%08x>' % dest)
		else:
			print(line)
		#end if
	else:
		addr = addr_re.match(line)
		if not addr:
			# Not a line of assembly
			print(line)
			continue
		#end if
		addr = int(addr.group(1), 16)
		if addr in funcs:
			# Function headers add a lot.
			print('sub_%08x:' % addr)
		#end if
		print(line)
	#end if
#next line

